<?php

// dépendances
require_once "impots_05_dao.php";

// ------------------------------------------------------------------

// interface metier
interface IImpotsMetier {
  public function calculerImpot($marié, $enfants, $salaire);
}

// ------------------------------------------------------------------
// classe d'implémentation
class ImpotsMetier implements IImpotsMetier {

  // couche dao
  private $dao;
  private $data;

  // getter et setter
  public function getDao() {
    return $this->dao;
  }

  public function setDao($dao) {
    $this->dao = $dao;
  }

  public function setData($data){
    $this->data=$data;
  }
  
  public function __construct($dao) {
    // on récupère les données nécessaires au calcul de l'impôt
    $this->setDao($dao);
    $this->setData($this->dao->getData());
  }

  public function calculerImpot($marié, $enfants, $salaire) {
    // $marié : oui, non
    // $enfants : nombre d'enfants
    // $salaire : salaire annuel
    // nombre de parts
    $marié = strtolower($marié);
    if ($marié == "oui")
      $nbParts = $enfants / 2 + 2;
    else
      $nbParts=$enfants / 2 + 1;
    // une 1/2 part de plus si au moins 3 enfants
    if ($enfants >= 3)
      $nbParts+=0.5;
    // revenu imposable
    $revenuImposable = 0.72 * $salaire;
    // quotient familial
    $quotient = $revenuImposable / $nbParts;
    // est mis à la fin du tableau limites pour arrêter la boucle qui suit
    $N = count($this->data);
    $this->data[$N - 1]->setLimite($quotient);
    // calcul de l'impôt
    $i = 0;
    while ($i < $N and $quotient > $this->data[$i]->getLimite()) {
      $i++;
    }
    // du fait qu'on a placé $quotient à la fin du tableau $limites, la boucle précédente
    // ne peut déborder du tableau $limites
    // maintenant on peut calculer l'impôt
    return floor($revenuImposable * $this->data[$i]->getCoeffR() - $nbParts * $this->data[$i]->getCoeffN());
  }

}