<?php

// connexion à la base MySql
// l'identité de l'utilisateur
$ID = "root";
$PWD = "";
// identité de la base
$DSN = "mysql:host=localhost;dbname=dbpersonnes";
// identité du fichier texte des commandes SQL à exécuter
$TEXTE = "creation.txt";

// connexion
list($erreur, $connexion) = connecte($DSN, $ID, $PWD);
if ($erreur) {
    print "Erreur lors de la connexion à la base [$DSN] sous l'identité ($ID,$PWD) : $erreur\n";
    exit;
}

// création et remplissage de la table
$erreurs = exécuterCommandes($connexion, $TEXTE, 1, 0);
//affichage nombre d'erreurs
print "il y a eu $erreurs[0] erreurs\n";
for ($i = 1; $i < count($erreurs); $i++)
    print "$erreurs[$i]\n";

// on se déconnecte et on quitte
déconnecte($connexion);
exit;

// ---------------------------------------------------------------------------------
function connecte($dsn, $login, $pwd) {
    // connecte ($login,$pwd) à la base $dsn
    // rend l'id de la connexion ainsi qu'un msg d'erreur
    try {
        // connexion
        $dbh = @new PDO($dsn, $login, $pwd, array(PDO::ATTR_PERSISTENT => true));
        // retour sans erreur
        return array("", $dbh);
    } catch (PDOException $e) {
        // retour avec erreur
        return array($e->getMessage(), null);
    }
}

//connecte
// ---------------------------------------------------------------------------------
function déconnecte($connexion) {
    // ferme la connexion identifiée par $connexion
    $connexion = null;
}

// ---------------------------------------------------------------------------------
function exécuteRequête($connexion, $sql) {
    // exécute la requête $sql sur la connexion $connexion
    // rend 1 msg d'erreur si erreur, la chaîne vide sinon
    try {
        $connexion->exec($sql);
        // retour sans erreur
        return "";
    } catch (PDOException $e) {
        // retour avec erreur
        return $e->getMessage();
    }
}

// ---------------------------------------------------------------------------------
function exécuterCommandes($connexion, $SQL, $suivi=0, $arrêt=1) {
    // utilise la connexion $connexion
    // exécute les commandes SQL contenues dans le fichier texte $SQL
    // ce fichier est un fichier de commandes SQL à exécuter à raison d'une par ligne
    // si $suivi=1 alors chaque exécution d'un ordre SQL fait l'objet d'un affichage indiquant sa réussite ou son échec
    // si $arrêt=1, la fonction s'arrête sur la 1ère erreur rencontrée sinon elle exécute ttes les commandes sql
    // la fonction rend un tableau (nb d'erreurs, erreur1, erreur2, ...)
    // on vérifie la présence du fichier $SQL

    if (!file_exists($SQL))
        return array(1, "Le fichier $SQL n'existe pas");

    // exécution des requêtes SQL contenues dans $SQL
    // on les met dans un tableau
    $requêtes = file($SQL);
    // on les exécute - au départ pas d'erreurs
    $erreurs = array(0);
    for ($i = 0; $i < count($requêtes); $i++) {
        //a-t-on une requête vide
        if (preg_match("/^\s*$/", $requêtes[$i]))
            continue;
        // exécution de la requête $i
        $erreur = exécuteRequête($connexion, $requêtes[$i]);
        //y-a-t-il eu une erreur ?
        if ($erreur) {
            // une erreur de plus
            $erreurs[0]++;
            // msg d'erreur
            $msg = "$requêtes[$i] : Erreur ($erreur)\n";
            $erreurs[] = $msg;
            // suivi écran ou non ?
            if ($suivi)
                print "$msg\n";
            // on s'arrête ?
            if ($arrêt)
                return $erreurs;
        } else {
            if ($suivi)
                print "$requêtes[$i] : Exécution réussie\n";
        }
    }//for
    // retour
    return $erreurs;
}
