<?php

// client impôts
// gestion des erreurs
ini_set("error_reporting", E_ALL & ~ E_WARNING & ~E_DEPRECATED & ~E_NOTICE);
ini_set("display_errors", "off");

// ---------------------------------------------------------------------------------
// une classe de fonctions utilitaires
class Utilitaires {

  function cutNewLinechar($ligne) {
    // on supprime la marque de fin de ligne de $ligne si elle existe
    $L = strlen($ligne);  // longueur ligne
    while (substr($ligne, $L - 1, 1) == "\n" or substr($ligne, $L - 1, 1) == "\r") {
      $ligne = substr($ligne, 0, $L - 1);
      $L--;
    }//while
    // fin
    return($ligne);
  }

}

// main -----------------------------------------------------
// définition des constantes
$DATA = "data.txt";
$RESULTATS = "resultats.txt";
// données serveur
$HOTE = "localhost";
$PORT = 80;
$urlServeur = "/exemples-web/impots_05B_web.php";

// les données nécessaires au calcul de l'impôt ont été placées dans la table mysqL IMPOTS
// appartenant à la base $BASE. La table a la structure suivante
// limites decimal(10,2), coeffR decimal(6,2), coeffN decimal(10,2)
// les paramètres des personnes imposables (statut marital, nombre d'enfants, salaire annuel)
// ont été placés dans le fichier texte $DATA à raison d'une ligne par contribuable
// les résultats (statut marital, nombre d'enfants, salaire annuel, impôt à payer) sont placés dans
// le fichier texte $RESULTATS à raison d'un résultat par ligne
// classe Utilitaires
$u = new Utilitaires();

// ouverture fichier des données des contribuables
$data = fopen($DATA, "r");
if (!$data) {
  print "Impossible d'ouvrir en lecture le fichier des données [$DATA]\n";
  exit;
}

// ouverture fichier des résultats
$résultats = fopen($RESULTATS, "w");
if (!$résultats) {
  print "Impossible de créer le fichier des résultats [$RESULTATS]\n";
  exit;
}

// on exploite la ligne courante du fichier des données contribuables
while ($ligne = fgets($data, 100)) {
  // on enlève l'éventuelle marque de fin de ligne
  $ligne = $u->cutNewLineChar($ligne);
  // on récupère les 3 champs marié:enfants:salaire qui forment $ligne
  list($marié, $enfants, $salaire) = explode(",", $ligne);
  // on calcule l'impôt
  list($erreur, $impôt) = calculerImpot($HOTE, $PORT, $urlServeur, $cookie, array($marié, $enfants, $salaire));
  // on inscrit le résultat
  $résultat = $erreur ? "$marié:$enfants:$salaire:$erreur" : "$marié:$enfants:$salaire:$impôt";
  fputs($résultats, "$résultat\n");
  // donnée suivante
}
// on ferme les fichiers
fclose($data);
fclose($résultats);

// fin
print "Terminé...\n";
exit;

function calculerImpot($HOTE, $PORT, $urlServeur, &$cookie, $params) {
  // connecte le client à ($HOTE,$PORT,$urlServeur)
// envoie le cookie $cookie si celui-ci est non vide. $cookie est passé par référence
// envoie $params au serveur
// affiche la ligne du résultat
// ouverture d'une connexion sur le port 80 de $HOTE
  $connexion = fsockopen($HOTE, $PORT);
  // erreur ?
  if (!$connexion)
    return array("erreur lors de la connexion au serveur ($HOTE, $PORT)");
  // les entêtes (headers) du protocole HTTP doivent se terminer par une ligne vide
  // GET
  fputs($connexion, "POST $urlServeur HTTP/1.1\n");
  // Host
  fputs($connexion, "Host: localhost\n");
  // Connection
  fputs($connexion, "Connection: close\n");
  // on envoie le cookie s'il est non vide
  if ($cookie) {
    fputs($connexion, "Cookie: $cookie\n");
  }//if

  // maintenant on envoie l'instruction client après l'avoir encodée
  $infos = "params=" . urlencode(implode(",", $params));
  // on indique quel type d'informations on va envoyer
  fputs($connexion, "Content-type: application/x-www-form-urlencoded\n");
  // on envoie la taille (nombre de caractères) des infos qui vont être envoyées
  fputs($connexion, "Content-length: " . strlen($infos) . "\n");
  // on envoie une ligne vide
  fputs($connexion, "\n");
  // on envoie les infos
  fputs($connexion, $infos);
  // on affiche la réponse du serveur web
  // et on prend soin de récupérer l'éventuel cookie
  while ($ligne = fgets($connexion, 1000)) {
    // cookie - seulement lors de la 1ère réponse
    if (!$cookie) {
      if (preg_match("/^Set-Cookie: (.*?)\s*$/", $ligne, $champs)) {
        $cookie = $champs[1];
      }//if
    }
    // dès qu'on a une ligne vide la réponse Http est terminée
    if (trim($ligne) == "") {
      break;
    }
  }//while
  // lecture ligne du résultat
  $ligne = fgets($connexion, 1000);
  // on ferme la connexion
  fclose($connexion);
  // calcul résultat
  $xml = new SimpleXMLElement($ligne);
  $erreur = isset($xml->erreur) ? $xml->erreur : "";
  $impôt = isset($xml->impot) ? $xml->impot : "";
  // retour
  return array($erreur, $impôt);
}
