<?php

// définition des constantes
$DATA = "data.txt";
$RESULTATS = "resultats.txt";
$IMPOTS = "impots.txt";

// les données nécessaires au calcul de l'impôt ont été placées dans le fichier IMPOTS
// à raison d'une ligne par tableau sous la forme
// val1:val2:val3,...
list($erreur, $limites, $coeffR, $coeffN) = getTables($IMPOTS);

// y-a-t-il eu une erreur ?
if ($erreur) {
  print "$erreur\n";
  exit;
}//if
// lecture des données utilisateur
$data = @fopen($DATA, "r");
if (!$data) {
  print "Impossible d'ouvrir en lecture le fichier des données [$DATA]\n";
  exit;
}

// ouverture fichier des résultats
$résultats = @fopen($RESULTATS, "w");
if (!$résultats) {
  print "Impossible de créer le fichier des résultats [$RESULTATS]\n";
  exit;
}

// on exploite la ligne courante du fichier des données
while ($ligne = fgets($data, 100)) {
  // on enlève l'éventuelle marque de fin de ligne
  $ligne = cutNewLineChar($ligne);
  // on récupère les 3 champs marié:enfants:salaire qui forment $ligne
  list($marié, $enfants, $salaire) = explode(",", $ligne);
  // on calcule l'impôt
  $impôt = calculImpots($marié, $enfants, $salaire, $limites, $coeffR, $coeffN);
  // on inscrit le résultat
  fputs($résultats, "$marié:$enfants:$salaire:$impôt\n");
  // donnée suivante
}// while
// on ferme les fichiers
fclose($data);
fclose($résultats);

// fin
exit;

// --------------------------------------------------------------------------
function cutNewLinechar($ligne) {
  // on supprime la marque de fin de ligne de $ligne si elle existe
  $L = strlen($ligne);  // longueur ligne
  while (substr($ligne, $L - 1, 1) == "\n" or substr($ligne, $L - 1, 1) == "\r") {
    $ligne = substr($ligne, 0, $L - 1);
    $L--;
  }
  // fin
  return($ligne);
}

// --------------------------------------------------------------------------
function calculImpots($marié, $enfants, $salaire, $limites, $coeffR, $coeffN) {
  // $marié : oui, non
  // $enfants : nombre d'enfants
  // $salaire : salaire annuel
  // $limites, $coeffR, $coeffN : les tableaux des données permettant le calcul de l'impôt
  // nombre de parts
  $marié = strToLower($marié);
  if ($marié == "oui")
    $nbParts = $enfants / 2 + 2;
  else
    $nbParts=$enfants / 2 + 1;
  // une 1/2 part de plus si au moins 3 enfants
  if ($enfants >= 3)
    $nbParts+=0.5;
  // revenu imposable
  $revenuImposable = 0.72 * $salaire;
  // quotient familial
  $quotient = $revenuImposable / $nbParts;
  // est mis à la fin du tableau limites pour arrêter la boucle qui suit
  $limites[count($limites)-1]=$quotient;
  // calcul de l'impôt
  $i = 0;
  while ($quotient > $limites[$i])
    $i++;
  // du fait qu'on a placé $quotient à la fin du tableau $limites, la boucle précédente
  // ne peut déborder du tableau $limites
  // maintenant on peut calculer l'impôt
  return floor($revenuImposable * $coeffR[$i] - $nbParts * $coeffN[$i]);
}

// --------------------------------------------------------------------------
function getTables($IMPOTS) {
  // $IMPOTS : le nom du fichier contenant les données des tables $limites, $coeffR, $coeffN
  // le fichier $IMPOTS existe-t-il ?
  if (!file_exists($IMPOTS))
    return array("Le fichier $IMPOTS n'existe pas","","","");
  // lecture en bloc du fichier
  $tables = file($IMPOTS);
  if (!$tables)
    return array("Erreur lors de l'exploitation du fichier $IMPOTS","","","");
  // création des 3 tables - on suppose que les lignes sont syntaxiquement correctes
  $limites = explode(":", cutNewLineChar($tables[0]));
  $coeffR = explode(":", cutNewLineChar($tables[1]));
  $coeffN = explode(":", cutNewLineChar($tables[2]));
  // fin
  return array("", $limites, $coeffR, $coeffN);
}
?>