<?php

// définition d'une classe Impôts
class Impôts {

  // attributs : les 3 tableaux de données
  private $limites;
  private $coeffR;
  private $coeffN;
  
  // constructeur

  public function __construct($IMPOTS) {
    // $IMPOTS : le nom du fichier contenant les données des tables $limites, $coeffR, $coeffN
    // le fichier $IMPOTS existe-t-il ?
    if (!file_exists($IMPOTS)) {
      throw new Exception("Le fichier $IMPOTS n'existe pas");
    }//if
    // lecture en bloc du fichier
    $tables = file($IMPOTS);
    if (!$tables) {
      throw new Exception("Erreur lors de l'exploitation du fichier $IMPOTS");
    }//if
    // création des 3 tables - on suppose que les lignes sont syntaxiquement correctes
    $u = new Utilitaires(); // pour disposer de fonctions utilitaires
    $this->limites = explode(":", $u->cutNewLineChar($tables[0]));
    $this->coeffR = explode(":", $u->cutNewLineChar($tables[1]));
    $this->coeffN = explode(":", $u->cutNewLineChar($tables[2]));
    }

  // --------------------------------------------------------------------------
  public function calculer($marié, $enfants, $salaire) {
    // $marié : oui, non
    // $enfants : nombre d'enfants
    // $salaire : salaire annuel
    
    // nombre de parts
    $marié = strtolower($marié);
    if ($marié == "oui")
      $nbParts = $enfants / 2 + 2;
    else
      $nbParts=$enfants / 2 + 1;
    // une 1/2 part de plus si au moins 3 enfants
    if ($enfants >= 3)
      $nbParts+=0.5;
    // revenu imposable
    $revenuImposable = 0.72 * $salaire;
    // quotient familial
    $quotient = $revenuImposable / $nbParts;
    // est mis à la fin du tableau limites pour arrêter la boucle qui suit
    $this->limites[count($this->limites) - 1] = $quotient;
    // calcul de l'impôt
    $i = 0;
    while ($quotient > $this->limites[$i])
      $i++;
    // du fait qu'on a placé $quotient à la fin du tableau $limites, la boucle précédente
    // ne peut déborder du tableau $limites
    // maintenant on peut calculer l'impôt
    return floor($revenuImposable * $this->coeffR[$i] - $nbParts * $this->coeffN[$i]);
  }

}

// une classe de fonctions utilitaires
class Utilitaires {

  public function cutNewLinechar($ligne) {
    // on supprime la marque de fin de ligne de $ligne si elle existe
    $L = strlen($ligne);  // longueur ligne
    while (substr($ligne, $L - 1, 1) == "\n" or substr($ligne, $L - 1, 1) == "\r") {
      $ligne = substr($ligne, 0, $L - 1);
      $L--;
    }//while
    // fin
    return($ligne);
  }

}

// test -----------------------------------------------------
// définition des constantes
$DATA = "data.txt";
$RESULTATS = "resultats.txt";
$IMPOTS = "impots.txt";
// les données nécessaires au calcul de l'impôt ont été placées dans le fichier IMPOTS
// à raison d'une ligne par tableau sous la forme
// "val1":"val2":"val3",...
// on crée un objet Impôts
try {
  $I = new Impôts($IMPOTS);
} catch (Exception $e) {
  print $e->getMessage();
  exit();
}
// on crée un objet utilitaires
$u = new Utilitaires();

// lecture des données utilisateur
$data = fopen($DATA, "r");
if (!$data) {
  print "Impossible d'ouvrir en lecture le fichier des données [$DATA]\n";
  exit;
}

// ouverture fichier des résultats
$résultats = fopen($RESULTATS, "w");
if (!$résultats) {
  print "Impossible de créer le fichier des résultats [$RESULTATS]\n";
  exit;
}

// on exploite la ligne courante du fichier des données
while ($ligne = fgets($data, 100)) {
  // on enlève l'éventuelle marque de fin de ligne
  $ligne = $u->cutNewLineChar($ligne);
  // on récupère les 3 champs marié:enfants:salaire qui forment $ligne
  list($marié, $enfants, $salaire) = explode(",", $ligne);
  // on calcule l'impôt
  $impôt = $I->calculer($marié, $enfants, $salaire);
  // on inscrit le résultat
  fputs($résultats, "$marié:$enfants:$salaire:$impôt\n");
  // donnée suivante
}// while
// on ferme les fichiers
fclose($data);
fclose($résultats);

// fin
print "Terminé\n";
exit;
?>
