<?php

// test -----------------------------------------------------
// définition des constantes
$DATA = "data.txt";
$RESULTATS = "resultats.txt";
$TABLE = "impots";
$BASE = "dbimpots";
$USER = "root";
$PWD = "";
$HOTE="localhost";

// les données nécessaires au calcul de l'impôt ont été placées dans la table mysqL $TABLE
// appartenant à la base $BASE. La table a la structure suivante
// limites decimal(10,2), coeffR decimal(6,2), coeffN decimal(10,2)
// les paramètres des personnes imposables (statut marital, nombre d'enfants, salaire annuel)
// ont été placés dans le fichier texte $DATA à raison d'une ligne par contribuable
// les résultats (statut marital, nombre d'enfants, salaire annuel, impôt à payer) sont placés dans
// le fichier texte $RESULTATS à raison d'un résultat par ligne
// on crée un objet Impôts
$I = new Impôts($HOTE, $USER, $PWD, $BASE, $TABLE);
// y-a-t-il eu une erreur ?
$erreur=$I->getErreur();
if ($erreur) {
  print "$I->erreur\n";
  exit;
}//if
// on crée un objet utilitaires
$u = new Utilitaires();

// ouverture fichier des données des contribuables
$data = @fopen($DATA, "r");
if (!$data) {
  print "Impossible d'ouvrir en lecture le fichier des données [$DATA]\n";
  exit;
}

// ouverture fichier des résultats
$résultats = @fopen($RESULTATS, "w");
if (!$résultats) {
  print "Impossible de créer le fichier des résultats [$RESULTATS]\n";
  exit;
}

// on exploite la ligne courante du fichier des données contribuables
while ($ligne = fgets($data, 100)) {
  // on enlève l'éventuelle marque de fin de ligne
  $ligne = $u->cutNewLineChar($ligne);
  // on récupère les 3 champs marié:enfants:salaire qui forment $ligne
  list($marié, $enfants, $salaire) = explode(",", $ligne);
  // on calcule l'impôt
  $impôt = $I->calculer($marié, $enfants, $salaire);
  // on inscrit le résultat
  fputs($résultats, "$marié:$enfants:$salaire:$impôt\n");
  // donnée suivante
}
// on ferme les fichiers
fclose($data);
fclose($résultats);

// fin
exit;

// ---------------------------------------------------------------------------------
// définition d'une classe Impôts
class Impôts {

  // attributs : les 3 tableaux de données
  private $limites;
  private $coeffR;
  private $coeffN;
  private $erreur;
  private $nbLimites;

  // getter
  public function getErreur(){
    return $this->erreur;
  }
  // constructeur

  function __construct($HOTE, $USER, $PWD, $BASE, $TABLE) {
    // initialise les attributs $limites, $coeffR, $coeffN
    // les données nécessaires au calcul de l'impôt ont été placées dans la table mysqL $TABLE
    // appartenant à la base $BASE. La table a la structure suivante
    // limites decimal(10,2), coeffR decimal(6,2), coeffN decimal(10,2)
    // la connexion à la base mysql de la machine $HOTE se fait sous l'identité ($USER,$PWD)
    // initialise le champ $erreur avec une éventuelle erreur
    // vide si pas d'erreur
    // 
    // connexion à la base mysql
    $DSN = "mysql:host=$HOTE;dbname=$BASE";
    list($erreur, $connexion) = connecte($DSN, $USER, $PWD);
    if ($erreur) {
      $this->erreur = "Erreur lors de la connexion à MySql sous l'identité ($HOTE,$USER,$PWD) : $erreur\n";
      return;
    }
    // lecture de la table $TABLE
    $requête = "select limites,coeffR,coeffN from $TABLE";
    // exécute la requête $requête sur la connexion $connexion
    try {
      $statement = $connexion->prepare($requête);
      $statement->execute();
      // exploitation du résultat de la requête
      while ($colonnes = $statement->fetch()) {
        $this->limites[] = $colonnes[0];
        $this->coeffR[] = $colonnes[1];
        $this->coeffN[] = $colonnes[2];
      }
      // déconnexion
      déconnecte($connexion);
      // pas d'erreur
      $this->erreur = "";
      // nombre d'élements de limites
      $this->nbLimites = count($this->limites);
    } catch (PDOException $e) {
      $this->erreur = $e->getMessage();
    }
  }
  // --------------------------------------------------------------------------
  function calculer($marié, $enfants, $salaire) {
    // $marié : oui, non
    // $enfants : nombre d'enfants
    // $salaire : salaire annuel
    // l'objet est-il dans un état correct ?
    if ($this->erreur)
      return -1;

    // nombre de parts
    $marié = strtolower($marié);
    if ($marié == "oui")
      $nbParts = $enfants / 2 + 2;
    else
      $nbParts=$enfants / 2 + 1;
    // une 1/2 part de plus si au moins 3 enfants
    if ($enfants >= 3)
      $nbParts+=0.5;
    // revenu imposable
    $revenuImposable = 0.72 * $salaire;
    // quotient familial
    $quotient = $revenuImposable / $nbParts;
    // est mis à la fin du tableau limites pour arrêter la boucle qui suit
    $N = $this->nbLimites;
    $this->limites[$N - 1] = $quotient;
    // calcul de l'impôt
    $i = 0;
    while ($i < $this->nbLimites and $quotient > $this->limites[$i])
      $i++;
    // du fait qu'on a placé $quotient à la fin du tableau $limites, la boucle précédente
    // ne peut déborder du tableau $limites
    // maintenant on peut calculer l'impôt
    return floor($revenuImposable * $this->coeffR[$i] - $nbParts * $this->coeffN[$i]);
  }

}

// ---------------------------------------------------------------------------------
// une classe de fonctions utilitaires
class Utilitaires {

  function cutNewLinechar($ligne) {
    // on supprime la marque de fin de ligne de $ligne si elle existe
    $L = strlen($ligne);  // longueur ligne
    while (substr($ligne, $L - 1, 1) == "\n" or substr($ligne, $L - 1, 1) == "\r") {
      $ligne = substr($ligne, 0, $L - 1);
      $L--;
    }//while
    // fin
    return($ligne);
  }

}
// ---------------------------------------------------------------------------------
function connecte($dsn, $login, $pwd) {
  // connecte ($login,$pwd) à la base $dsn
  // rend l'id de la connexion ainsi qu'un msg d'erreur
  try {
    // connexion
    $dbh = @new PDO($dsn, $login, $pwd, array(PDO::ATTR_PERSISTENT => true));
    // retour sans erreur
    return array("", $dbh);
  } catch (PDOException $e) {
    // retour avec erreur
    return array($e->getMessage(), null);
  }
}

//connecte
// ---------------------------------------------------------------------------------
function déconnecte($connexion) {
  // ferme la connexion identifiée par $connexion
  $connexion = null;
}


?>
