<?php

// transfère le fichier texte des données nécessaires au calcul des impôts
// dans une table mysql
// les données
$IMPOTS = "impots.txt";
$BASE = "dbimpots";
$TABLE = "impots";
$USER = "root";
$PWD = "";
$HOTE = "localhost";

// les données nécessaires au calcul de l'impôt ont été placées dans le fichier $IMPOTS
// à raison d'une ligne par tableau sous la forme
// val1:val2:val3,...
list($erreur, $limites, $coeffR, $coeffN) = getTables($IMPOTS);
// y-a-t-il eu une erreur ?
if ($erreur) {
  print "$erreur\n";
  exit;
}//if
// on transfère ces tableaux dans une table mysql
$erreur = copyToMysql($limites, $coeffR, $coeffN, $HOTE, $USER, $PWD, $BASE, $TABLE);
if ($erreur)
  print "$erreur\n";
else
  print "Transfert opéré\n";
// fin
exit;

// --------------------------------------------------------------------------
function copyToMysql($limites, $coeffR, $coeffN, $HOTE, $USER, $PWD, $BASE, $TABLE) {
  // copie les 3 tableaux numériques $limites, $coeffR, $coeffN
  // dans la table $TABLE de la base mysql $BASE
  // la base mysql est sur la machine $HOTE
  // l'utilisateur est identifié par $USER et $PWD
  // connexion
  list($erreur, $connexion) = connecte("mysql:host=$HOTE;dbname=$BASE", $USER, $PWD);
  if ($erreur)
    return "Erreur lors de la connexion à MySql sous l'identité ($HOTE,$USER,$PWD) : $erreur\n";
  // suppression de la table
  $requête = "drop table $TABLE";
  exécuteRequête($connexion, $requête);
  // création de la table
  $requête = "create table $TABLE (limites decimal(10,2), coeffR decimal(6,2), coeffN decimal(10,2))";
  list($erreur, $res) = exécuteRequête($connexion, $requête);
  if ($erreur)
    return "$requête : erreur ($erreur)";
  // remplissage
  for ($i = 0; $i < count($limites); $i++) {
    // requête d'insertion
    $requête = "insert into $TABLE (limites,coeffR,coeffN) values ($limites[$i],$coeffR[$i],$coeffN[$i])";
    // exécution de la requête
    list($erreur, $res) = exécuteRequête($connexion, $requête);
    // retour si erreur
    if ($erreur)
      return "$requête : erreur ($erreur)";
  }//for
  // c'est fini - on se déconnecte
  déconnecte($connexion);
  // retour sans erreur
  return "";
}
// --------------------------------------------------------------------------
function getTables($IMPOTS) {
  // $IMPOTS : le nom du fichier contenant les données des tables $limites, $coeffR, $coeffN
  // le fichier $IMPOTS existe-t-il ?
  if (!file_exists($IMPOTS))
    return array("Le fichier $IMPOTS n'existe pas");
  // lecture en bloc du fichier
  $tables = file($IMPOTS);
  if (!$tables)
    return array("Erreur lors de l'exploitation du fichier $IMPOTS");
  // création des 3 tables - on suppose que les lignes sont syntaxiquement correctes
  $limites = explode(":", cutNewLineChar($tables[0]));
  $coeffR = explode(":", cutNewLineChar($tables[1]));
  $coeffN = explode(":", cutNewLineChar($tables[2]));
  // fin
  return array("", $limites, $coeffR, $coeffN);
}
// --------------------------------------------------------------------------
function cutNewLinechar($ligne) {
  // on supprime la marque de fin de ligne de $ligne si elle existe
  $L = strlen($ligne);  // longueur ligne
  while (substr($ligne, $L - 1, 1) == "\n" or substr($ligne, $L - 1, 1) == "\r") {
    $ligne = substr($ligne, 0, $L - 1);
    $L--;
  }
  // fin
  return($ligne);
}
// ---------------------------------------------------------------------------------
function connecte($dsn, $login, $pwd) {
  // connecte ($login,$pwd) à la base $dsn
  // rend l'id de la connexion ainsi qu'un msg d'erreur
  try {
    // connexion
    $dbh = @new PDO($dsn, $login, $pwd, array(PDO::ATTR_PERSISTENT => true));
    // retour sans erreur
    return array("", $dbh);
  } catch (PDOException $e) {
    // retour avec erreur
    return array($e->getMessage(), null);
  }
}

//connecte
// ---------------------------------------------------------------------------------
function déconnecte($connexion) {
  // ferme la connexion identifiée par $connexion
  $connexion = null;
}

// ---------------------------------------------------------------------------------
function exécuteRequête($connexion, $sql) {
  // exécute la requête $sql sur la connexion $connexion
  // rend un tableau de 2 éléments ($erreur,$résultat)
  // on détermine si c'est un select ou non
  $commande = "";
  if (preg_match("/^\s*(\S+)/", $sql, $champs)) {
    $commande = $champs[0];
  }
  // exécution de la commande
  try {
    if (strtolower($commande) == "select") {
      $res = $connexion->query($sql);
    } else {
      $res = $connexion->exec($sql);
      if ($res === FALSE) {
        $info = $connexion->errorInfo();
        return array($info[2], null);
      }
    }
    // retour sans erreur
    return array("", $res);
  } catch (PDOException $e) {
    // retour avec erreur
    return array($e->getMessage(), null);
  }
}
